/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.linux.usermanager.j2n;

import com.smardec.demo.j2native.linux.usermanager.info.GroupInfo;
import com.smardec.demo.j2native.linux.usermanager.info.UserInfo;
import com.smardec.demo.j2native.tools.LibraryManager;
import com.smardec.demo.j2native.tools.NativeLibraryHelper;
import com.smardec.j2native.Function;
import com.smardec.j2native.Library;
import com.smardec.j2native.Pointer;
import com.smardec.j2native.UInt;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;


public class InfoProvider {
    private static InfoProvider instance;
    private Function setpwent;
    private Function getpwent;
    private Function endpwent;
    private Function setgrent;
    private Function getgrent;
    private Function endgrent;
    private Function getpwnam;
    private Function getuid;
    private Function getpwuid;
    private Function getspnam;
    private Function setspent;
    private Function getspent;
    private Function endspent;

    private HashMap groups4user;
    private HashMap users4DefaultGroup;
    private List users;
    private List groups;

    private InfoProvider() {
        Library kernel = LibraryManager.getInstance().getLibrary(NativeLibraryHelper.xNIX_KERNEL_NAME);
        setpwent = kernel.getFunction("setpwent");
        getpwent = kernel.getFunction("getpwent");
        endpwent = kernel.getFunction("endpwent");
        setgrent = kernel.getFunction("setgrent");
        getgrent = kernel.getFunction("getgrent");
        endgrent = kernel.getFunction("endgrent");
        getpwnam = kernel.getFunction("getpwnam");
        getpwuid = kernel.getFunction("getpwuid");
        getspnam = kernel.getFunction("getspnam");
        setspent = kernel.getFunction("setspent");
        getspent = kernel.getFunction("getspent");
        endspent = kernel.getFunction("endspent");
        getuid = kernel.getFunction("getuid");

        groups4user = new HashMap();
        users4DefaultGroup = new HashMap();
        users = new ArrayList();
        groups = new ArrayList();
    }

    public static InfoProvider getInfoProvider() {
        if (instance == null) {
            instance = new InfoProvider();
        }
        return instance;
    }

    public void loadInfos() {
        loadGroups();
        loadUsers();
        tuneGroups();
        tuneUsers();
    }

    public List getAllUsers() {
        return users;
    }

    public List getAllGroups() {
        return groups;
    }

    public int getCurrentUserId() {
        UInt userId = new UInt(-1);
        getuid.invoke(userId);
        return userId.intValue();
    }

    private void loadGroups() {
        groups4user = new HashMap();
        groups = new ArrayList();
        setgrent.invoke(null);
        Group group = new Group();
        Pointer groupPtr = new Pointer(group);
        getgrent.invoke(groupPtr);
        while (!groupPtr.isNull()) {
            GroupInfo groupInfo = new GroupInfo(group.gr_gid().intValue(), group.gr_name().toString());
            groupInfo.setGroupPass(group.gr_passwd().toString());

            List groupMembers = new ArrayList();
            Pointer memberPointerPtr = group.gr_mem();
            int i = 0;
            Pointer memberPtr = (Pointer) memberPointerPtr.getAt(i++);
            while (!memberPtr.isNull()) {
                String userName = memberPtr.getReferencedArgument().toString();
                List userGroups = (List) groups4user.get(userName);
                if (userGroups == null) {
                    userGroups = new ArrayList();
                    groups4user.put(userName, userGroups);
                }
                userGroups.add(groupInfo);
                groupMembers.add(userName);
                memberPtr = (Pointer) memberPointerPtr.getAt(i++);
            }
            groupInfo.setGroupMembers(groupMembers);
            groups.add(groupInfo);

            getgrent.invoke(groupPtr);
        }
        endgrent.invoke(null);
    }

    private void loadUsers() {
        users4DefaultGroup = new HashMap();
        users = new ArrayList();
        setpwent.invoke(null);
        Passwd passwd = new Passwd();
        Spwd spwd = new Spwd();
        Pointer passwdPtr = new Pointer(passwd);
        Pointer spwdPtr = new Pointer(spwd);
        getpwent.invoke(passwdPtr);
        while (!passwdPtr.isNull()) {
            UserInfo userInfo = new UserInfo(passwd.pw_name().toString(), passwd.pw_uid().intValue());
            userInfo.setRealName(passwd.pw_gecos().toString());
            userInfo.setShellPrg(passwd.pw_shell().toString());
            userInfo.setHomeDir(passwd.pw_dir().toString());
            userInfo.setGroupId(passwd.pw_gid().intValue());

            List groupUsers = (List) users4DefaultGroup.get(new Integer(passwd.pw_gid().intValue()));
            if (groupUsers == null) {
                groupUsers = new ArrayList();
                users4DefaultGroup.put(new Integer(passwd.pw_gid().intValue()), groupUsers);
            }
            groupUsers.add(userInfo.getUserName());

            setspent.invoke(null);
            getspent.invoke(spwdPtr);
            while (!spwdPtr.isNull()) {
                if (spwd.sp_namp().equals(passwd.pw_name())) {
                    userInfo.setDateOfLastChange(spwd.sp_lstchg().intValue());
                    userInfo.setDaysExpire(spwd.sp_expire().intValue());
                    userInfo.setDaysInactive(spwd.sp_inact().intValue());
                    userInfo.setDaysToWarn(spwd.sp_warn().intValue());
                    userInfo.setMaxDaysChange(spwd.sp_max().intValue());
                    userInfo.setMinDaysChange(spwd.sp_min().intValue());
                    userInfo.setUserPass(spwd.sp_pwdp().toString());
                    break;
                }
                getspent.invoke(spwdPtr);
            }

//            getspnam.invoke(spwdPtr, new Pointer.Const(passwd.pw_name()));
            endspent.invoke(null);
//            if(!spwdPtr.isNull()){
//                userInfo.setDateOfLastChange(spwd.sp_lstchg().intValue());
//                userInfo.setDaysExpire(spwd.sp_expire().intValue());
//                userInfo.setDaysInactive(spwd.sp_inact().intValue());
//                userInfo.setDaysToWarn(spwd.sp_warn().intValue());
//                userInfo.setMaxDaysChange(spwd.sp_max().intValue());
//                userInfo.setMinDaysChange(spwd.sp_min().intValue());
//                userInfo.setUserPass(spwd.sp_pwdp().toString());
//            }
            // set of groups should be done later
            userInfo.setGroups(null);
            users.add(userInfo);

            getpwent.invoke(passwdPtr);
        }
        endpwent.invoke(null);
    }

    private void tuneGroups() {
        for (int i = groups.size() - 1; i >= 0; i--) {
            GroupInfo groupInfo = (GroupInfo) groups.get(i);
            List defUsers = (List) users4DefaultGroup.get(new Integer(groupInfo.getId()));
            if (defUsers != null) {
                List groupMembers = groupInfo.getGroupMembers();
                for (int j = defUsers.size() - 1; j >= 0; j--) {
                    if (!groupMembers.contains(defUsers.get(j))) {
                        groupMembers.add(defUsers.get(j));
                    }
                }

            }
        }
    }

    private void tuneUsers() {
        for (int i = users.size() - 1; i >= 0; i--) {
            UserInfo userInfo = (UserInfo) users.get(i);
            List userGroups = (List) groups4user.get(userInfo.getUserName());
            if (userGroups == null) {
                userGroups = new ArrayList();
            }
            for (int j = groups.size() - 1; j >= 0; j--) {
                GroupInfo groupInfo = (GroupInfo) groups.get(j);
                if ((groupInfo.getId() == userInfo.getGroupId()) && (!userGroups.contains(groupInfo))) {
                    userGroups.add(groupInfo);
                    break;
                }
            }
            userInfo.setGroups(userGroups);
        }
    }
}
